#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2003 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

use strict;

our (
	$VERSION, $platform, $srvname, $config, 
	$key, $Registry, $confdir
);

$VERSION = "1.4.0";
$platform = lc $^O; # linux, mswin32

use FindBin qw($Bin);
use File::Spec;
use File::Copy;
use File::Path;
use lib "$Bin/util/perl";
use AppConfig qw (:argcount);
use Crypt::RandPasswd;
use Crypt::Tea;

$srvname = "LogrepServer";
$key = 'LoGrEp 1901 LoGfIlE 1502 ExTrAcTiOn 2009';

# Create a config object, we require one parameter as default
$config = AppConfig->new ();

# define platform specific configration  and bin directories

$confdir = ($platform eq 'linux') ? "/etc/logrep" :
	   ($platform eq 'mswin32') ? "$Bin/../etc" : "";

# Define options
$config->define ("genkeys", 	{ DEFAULT => 0 } );
$config->define ("setperms", 	{ DEFAULT => 0 } );
$config->define ("genpwd", 	{ DEFAULT => 0 } );
$config->define ("makeclient", 	{ DEFAULT => "<undef>", ARGCOUNT => ARGCOUNT_ONE } );
$config->define ("keyfile", 	{ DEFAULT => "$confdir/logrep.key", ARGCOUNT => ARGCOUNT_ONE } );
$config->define ("user", 	{ DEFAULT => "logrep", ARGCOUNT => ARGCOUNT_ONE } );
$config->define ("pwdfile", 	{ DEFAULT => "$confdir/logrep.pwd", ARGCOUNT => ARGCOUNT_ONE } );
$config->define ("help", 	{ DEFAULT => 0, ALIAS => "h|?" } );

# Process command line arguments
$config->args ();

if ($config->help) {

	print 
"
logrep-admin.pl - administration/configuration interface for logrep server

  Options:
    --help 	- prints this message
    --genkeys	- generates a key pair for ssh authentication
    --setperms	- sets permissions on package contents
    --genpwd	- Generate password for logrep client user accounts
    --makeclient <linux | windows>
		- Makes a client package
";
	exit 0;
}

if ($config->genkeys) {

	# Create key files (with no passphrase!)
	system "\"$Bin/util/bin/ssh-keygen\" -q -b 1024 -t rsa -f \"" . $config->keyfile . "\" -N ''";

}

if ($config->setperms) {

	# Make sure that all system is only accessible by its user
	system "\"$Bin/util/bin/chmod\" -R -c 700 \"$Bin/..\"";

}

if ($config->genpwd) {

	# Generate a user password 
	my $password = 
		Crypt::RandPasswd->word (4,4) .
		Crypt::RandPasswd->rand_int_in_range (1000,9999) .
		Crypt::RandPasswd->word (4,4) .
		Crypt::RandPasswd->rand_int_in_range (1000,9999);

	# Save user password as an encrypted file
	my $encryptedtext = &encrypt ($password, $key);

	open PASSFILE, "> " . $config->pwdfile or die;
	print PASSFILE $encryptedtext;
	close PASSFILE;

}

if ($config->makeclient eq "linux" || $config->makeclient eq "windows") { # Make a client for distribution

	my $cltype = $config->makeclient;
	my $workdir = "logrep-$cltype-client-$VERSION";
	rmtree ($workdir) if -e $workdir;	# Remove existing working directory if exists
	mkdir $workdir or die;

	my $user = $config->user;
	my $group = ($cltype eq "linux") ? $user : "SYSTEM";
	my ($volume,$directories,$file) = File::Spec->splitpath( $config->keyfile );
	my $pubfile = $file . ".pub";

	my (undef,undef,undef,undef,undef,undef,undef,$size,undef,undef,undef,undef,undef)
           = stat($config->pwdfile);


	open PASSFILE, $config->pwdfile;	# Read encrypted user password from file
	my $encryptedpwd = '';
	read PASSFILE, $encryptedpwd, $size;
	close PASSFILE;

	my $password = &decrypt ($encryptedpwd, $key);

	open SCRIPTFILE, "> $workdir/install.sh" or die;

	print SCRIPTFILE "
#!/bin/bash
# Logrep $cltype client installation version $VERSION
echo Preparing your machine for Logrep ...
echo .
echo Adding user $user ..
";

	print SCRIPTFILE ($cltype eq "linux") ?
		"useradd -c 'Logrep user' $user" . "\n" .
		"echo $password | passwd --stdin $user > /dev/null" . "\n"
		:
		"net user $user $password /add /fullname:\"Logrep user\"" ."\n" .
		"mkpasswd -l -u $user  >> /etc/passwd" . "\n";


	print SCRIPTFILE "
echo Copying public key for SSH authentication ..
mkdir -p ~$user/.ssh
cat $pubfile > ~$user/.ssh/authorized_keys
cat $pubfile > ~$user/.ssh/authorized_keys2
echo Adjust ownership and permissions ..
chown $user:$group ~$user/.ssh ~$user/.ssh/authorized_keys ~$user/.ssh/authorized_keys2
chmod 755 ~$user/.ssh
chmod 644 ~$user/.ssh/authorized_keys ~$user/.ssh/authorized_keys2
echo .
echo D O N E
echo .
echo NB! This package contains password information.
echo PLEASE REMOVE IT FROM THIS MACHINE
echo .
";
#############
	close SCRIPTFILE;

	copy (File::Spec->catpath($volume,$directories,$pubfile), "$workdir/$pubfile");

	# Convert text before packing
	ConvertTextFile (($cltype eq "linux")?"2unix":"2dos", "$workdir/install.sh");

	# Make a tar archive
	system ("\"$Bin/util/bin/tar\" cf $workdir.tar $workdir");

	rmtree ($workdir);	# Remove existing working directory

	print "\u$cltype client package $workdir.tar is available for distribution now.\n";
}

###############
sub ConvertTextFile {

	my ($type, $file) = @_;

	my (undef,undef,undef,undef,undef,undef,undef,$size,undef,undef,undef,undef,undef)
           = stat($file);

	return 0 if ((! -e $file) || !(($type eq "2unix") || ($type eq "2dos")));

	my $buf = "";

	open TEXTFILE, $file or die;
	sysread TEXTFILE, $buf, $size;
	close TEXTFILE;

	$buf =~ s/\r\n/\n/g if ($type eq "2unix");
	$buf =~ s/\n/\r\n/g if ($type eq "2dos");
		
	open TEXTFILE, "> $file" or die;
	syswrite TEXTFILE, $buf, $size;
	close TEXTFILE;

	return 1;
}


