#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2003 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

###############
# Logrep daemon/service : picks log files for systems and generates reports
###############
use FindBin qw($Bin);
use strict "vars";

use CGI;
use lib "$Bin/util/perl";

use Logrep::Config qw (&LoadConfigFile &ExpandParVal &MakePathAvailable %client %favoriteset $config);
use Logrep::Common;

our ( 
	$q, $debug, $exitnow, $ARBITRARY_OFFSET, $DEBUG, %snapshotNextrun
);

$debug = $config->debug;

$exitnow = 0;	# run logrep as long as $exitnow is 0

# Make a daemon server if not debugging (linux only)
MakeDaemon () if ( (!$debug) && (lc($^O) eq 'linux'));

$q = new CGI ('');

$ARBITRARY_OFFSET = 5;	# to arbitratize report run times in seconds

LoadConfigFile ();

PrintLog  ("logrep-start", sprintf ("%d clients, %d favorite sets", scalar keys (%client), scalar keys (%favoriteset)));

#####################################################
# We have all information we need to start logrep facilities
#

# LOOP! LOOP! LOOP!

# Initialize nextrun counter array for snapshot reports
foreach my $lcln (keys %client) { $snapshotNextrun {$lcln} = 0 };

until ($exitnow) {

	my $nrep = 0; 	# number of reports generated in this cycle

	sleep int(rand($ARBITRARY_OFFSET));	# Arbitratize start time

	# Run a snapshot cycle
	foreach my $lcln (keys %client) {
		
		next if ExpandParVal($lcln, "snapshot_interval") == 0;	# skip if snapshot interval is 0

		# Skip if scheduled run time has not come yet !!
		next if (($snapshotNextrun {$lcln} > 0) && ($snapshotNextrun {$lcln} > time ()));

		PrintLog ("snapshot", "Generate client report for $lcln");
		GenerateClientReport ("snapshot", $lcln);	# Generate a snapshot report

		$nrep++;

		# schedule next run
		$snapshotNextrun {$lcln} = time () + (ExpandParVal ($lcln, "snapshot_interval") * 60); # In seconds

	}

	# Run a full report cycle
	foreach my $lcln (keys %client) {


		# next if no schedule is defined or there is something wrong with schedule parameter
		my $nextrun = CalcFullInterval (ExpandParVal ($lcln, "full_schedule"));

		print "Full schedule : " . localtime($nextrun) . ", Now : " . localtime () . "\n" if $debug;

		next if ($nextrun == -1 || ($nextrun > time ()));

		PrintLog ("full", "Generate client report for $lcln");
		GenerateClientReport ("full", $lcln);	# Generate a full report

		$nrep++;
	}

	# Start next cycle if there were no client reports generated in this cycle
	next if ($nrep == 0);

	# Run a favorite set cycle
	foreach my $lfavset (keys %favoriteset) {

		PrintLog ("favoriteset", "Generate report for $lfavset");
		GenerateFavsetReport ($lfavset);
	}

}

#######################################
#   GenerateClientReport : Generates a logrep report for a client
#   Accepts two arguments :
#       $reptype  : report type - "snapshot" or "full"
#       $lclient  : logrep client
#
sub GenerateClientReport {

	my ($reptype, $lclient) = @_;

	# Get and expand variables for remote command execution / report generation
	my $verbose	= ($debug) ? "-v" : "";
	my $type 	= ExpandParVal ($lclient, "type");
	my $privkey 	= ExpandParVal ($lclient, "keyfile");
	my $user	= ExpandParVal ($lclient, "localuser");
	my $ipadr	= ExpandParVal ($lclient, "ipaddress");
	my $remcmd	= ExpandParVal ($lclient, "$reptype" . "_command");
	my $logsfile	= ExpandParVal ($lclient, "$reptype" . "_server_logfile");
	my $webdir	= ExpandParVal ($lclient, "$reptype" . "_server_webpath");
	my $dimension	= (ExpandParVal ($lclient, "dimensions") == 0) ? "" : ("-d ". ExpandParVal ($lclient, "dimensions"));
	my $singlevec	= (ExpandParVal ($lclient, "singlevector") eq "on") ? "-r" : "";
	my $attributes	= (ExpandParVal ($lclient, "attributes") eq "*") ? "" : ("-s " . ExpandParVal ($lclient, "attributes"));

	MakePathAvailable ($logsfile); 	# Make server path available if necessary
	MakePathAvailable ($webdir);	# Make logrep report path available if necessary

	my $loggz .= $logsfile . ".gz";	# add gzip extension
	my $logtmp = $logsfile . ".tmp"; # Decompressed temporary log file

	# Run remote command that produces a gzipped remote file in user's home directory
	RunCommand ("\"$Bin/util/bin/ssh\" -l $user -i \"$privkey\" $ipadr \"$remcmd > $lclient\"");

	# Pick up this file via scp, cygwin scp requires special treatment of drive letters
	# Change drive name to /cygdrive/<drive name>
	my ($drive, $path) = ($loggz =~ /^([A-Z,a-z])\:[\/,\\](.*)/);
	my $scpdest = (defined $drive) ? "/cygdrive/$drive/$path" : $loggz;

	RunCommand ("\"$Bin/util/bin/scp\" -q -S \"$Bin/util/bin/ssh\" -i \"$privkey\" $user\@$ipadr:$lclient \"$scpdest\"");

	# Operation is unsuccessful if server log file does not exist or has a zero size
	return 0 if ((! -e "$loggz") || (-z "$loggz"));

	# Decompress log file
	RunCommand ("\"$Bin/util/bin/gzip\" -f -c -d \"$loggz\" > \"$logtmp\"");

	# Generate reports
	RunCommand ("perl \"$Bin/module-$type.pl\" $verbose -l \"$logtmp\" -w \"$webdir\" $dimension $attributes $singlevec");
	
 	unlink $logtmp;
}

###################################
#   GenerateFavsetReport : Generates a logrep report for a favorite set
#   Accepts one argument :
#       $lfavset  : logrep favorite set
#
sub GenerateFavsetReport {

	my $lfavset = shift;

	# Get and expand variables for favorite set generation
	my $favsfile	= ExpandParVal ("systemdefault", "favoriteset_server_webpath");

	$favsfile .= "/$lfavset.html";

	MakePathAvailable ($favsfile); 	# Make server path available if necessary

	open HTMLPAGE, "> $favsfile";

	my $hdrtext = GenerateHtmlHeader ();
	print HTMLPAGE $hdrtext;

	GenerateFavsetElement ("graph", 2, $lfavset, "Favorite graphs", HTMLPAGE);
	GenerateFavsetElement ("report", 4, $lfavset, "Favorite report", HTMLPAGE);
	GenerateFavsetElement ("favoriteset", 3, $lfavset, "Favorite sets", HTMLPAGE);
	GenerateFavsetElement ("client", 4, $lfavset, "Favorite clients", HTMLPAGE);

	print HTMLPAGE $q->end_html;
	close HTMLPAGE;

}

###################################
#    GenerateFavsetElement : Generates one of elements in a favorite set report
#    Accepts 5 arguments :
#        favsetelm - element type ('graph', 'report', 'favoriteset' or 'client')
#        maxcol - number of columns per row
#        lfavset - favorite set name
#        favtitle - element title
#        fp - file handle for favorite set report
#
sub GenerateFavsetElement {

	my ($favsetelm, $maxcol, $lfavset, $favtitle, $fp) = @_;

	my @favsetarr = ExpandParVal ($lfavset, $favsetelm, 1);

	# no operation if favorite set is nor defined or has no elements
	return 0 if (scalar @favsetarr < 1);
   
	my $ncols = 0;

	# print element header 
	print $fp $q->div ({align=>"center"}, $q->h3 ($favtitle) );
	print $fp '<div align="center"><table border="1" bordercolor="#800000">';

	foreach my $valstr (@favsetarr) {

		my @valarr = split (/\s+/, $valstr);

		# no operation if favorite set element has no parameters defined
		return if (scalar @valarr < 1);

		my ($cln, $reptype, $attribute) = @valarr;

		# We expect that at least client name/favorite set name is defined
		return if (! (exists $client{$cln} || exists $favoriteset{$cln}));

		my $sysname 	= ExpandParVal ($cln, "type");
		my $ipadr 	= ExpandParVal ($cln,"ipaddress");
		my $webdir	= ExpandParVal ($cln, "$reptype" . "_server_webpath");

		print $fp "<tr>" if (($ncols % $maxcol) == 0);
	
		FAVSETELM_SPEC: {
			
			# 'graph' specific operations
			if ($favsetelm eq "graph" && defined $attribute) {
				my $img 	= "$webdir/$attribute/$attribute.png";
				my $link 	= "$webdir/$attribute/index.html";

				last  FAVSETELM_SPEC if (!(-e $img && -e $link));

				print $fp
				  $q->td ( $q->strong (
				    $q->font ( {size=>"+1"}, "$sysname - $ipadr" ),
				    $q->br,
				    $q->a ({href=>$link}, $q->img ({src=>"$img"}))
				  ));

				last FAVSETELM_SPEC;
			};

			# 'report' specific operations
			if ($favsetelm eq "report" && defined $attribute) {
				my $link 	= "$webdir/$attribute/index.html";

				last  FAVSETELM_SPEC if (!(-e $link));

				print $fp
				  $q->td ( $q->strong (
				    $q->font ( {size=>"+1"}, $q->a ({href=>$link}, "$attribute ($sysname - $ipadr)"))
				  ));

				last FAVSETELM_SPEC;
			};

			# 'client' specific operations
			if ($favsetelm eq "client") {
				my $link = "$webdir/index.html";

				last  FAVSETELM_SPEC if (!(-e $link));

				print $fp
	  			  $q->td ( $q->strong (
				    $q->font ( {size=>"+1"}, $q->a ({href=>"$link"}, "$sysname - $ipadr"))
				  ));

				last FAVSETELM_SPEC;
			};

			# 'favoriteset' specific operations
			if ($favsetelm eq "favoriteset") {

				my $favdir = ExpandParVal ("systemdefault", "favoriteset_server_webpath");
				my $link = "$favdir/$cln.html";

				last  FAVSETELM_SPEC if (!(-e $link));

				print $fp
	  			  $q->td ( $q->strong (
				    $q->font ( {size=>"+1"}, $q->a ({href=>"$link"}, $cln))
				  ));

				last FAVSETELM_SPEC;
			};

		}

		$ncols++;

		print $fp "</tr>" if (($ncols % $maxcol) == 0);
	
	}	
		
	print $fp "</table></div>";
	print $fp	$q->p ( $q->hr );	

}

sub RunCommand {

	my $cmd = shift;
	print "Command: $cmd\n" if $debug;
	system $cmd;

}

sub CalcFullInterval {

	my $fullschval = lc shift;

	return -1 if ($fullschval eq "none");

	my ($schour, $scmin) = ( $fullschval =~ /^(\d+):(\d+)/);

	my ($sec,$min,$hour,$mday,$mon,undef,undef,undef,undef) = localtime ();

	# find hh:mm difference between now and scheduled time
	my $schdiff = (($schour - $hour) * 60 + ($scmin - $min));

	# make sure next day schedule if HH:MM diff is negative
	if ($schdiff < 0) { $schdiff += (24 * 60) }

	return (time () + ($schdiff * 60));
}

#######################################
#   PrintLog: prints logrep activity to a logfile
#   Requires two parameters:
#       class - log type
#	message - message
#
sub PrintLog {

	my ($class, $message) = @_;

	# Preparations for logrep logging
	my $logfile = ExpandParVal ("systemdefault", "logrep_logfile");
	
	# Make sure that path for logrep log file is available
	MakePathAvailable ($logfile);
	open LOGFILE, ">> $logfile";

	my ($sec,$min,$hour,$mday,$mon,$year,undef,undef,undef) = localtime ();

	my $timestamp = sprintf ("%04d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $mday, $hour, $min, $sec);

	print LOGFILE "$timestamp $class $message\n";

	close LOGFILE;

}

#######################################
#   MakeDaemon: runs logrep as a daemon (linux only)
#
sub MakeDaemon {

	my $pid = fork;
	exit if $pid;
	die "Couldn't fork: $!" unless defined ($pid);

	require POSIX;
	POSIX::setsid ()
		or die "Can't start a new session: $!";

	sub SignalHandler { $exitnow = 1 };

	$SIG{INT} = $SIG{TERM} = $SIG{HUP} = \&SignalHandler;
	$SIG{PIPE} = 'IGNORE';

}
