package Logrep::Common;

#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2003 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
# LOGREP - Log reporting system common library, jan 02, TK

=head1 NAME

Logrep::Common - Common library for logrep modules

=head1 SYNOPSIS

    use Logrep::Common;

    SetCommonParameters ();
    GenerateReport ();
    GenerateTables ();


    Exported variables :
        
        $config - pointer to logrep configuration object
        $dieopenmsg - common error message during file open
        $dieclosemsg - common error message during file close
        @dim - array containing attribute list. Must be set.
        $startdate - date from first log line (used in reports.)
	$enddate - date from last log line (used in reports.)
	%monthnr - lookup table (from MMM to month number)
	$n - number of lines in the processes log file. Must be set.

=head1 DESCRIPTION

This module contains common reporting, counting and analysis functions used by 
logrep parsing modules.

=cut

###########################################################

use strict;
use Exporter;
use FindBin qw($Bin);
use CGI qw/*table/;
use Cwd;

use lib "$Bin/..";
use Logrep::Config qw($config);
use GD::Graph::bars;
use Net::DNS;

# Module definition globals
#
our $VERSION = "1.0.0";
our @ISA = qw(Exporter);
our @EXPORT = qw (
	&SetCommonParameters
	&GenerateReport
	&GenerateTables
	&GenerateHtmlHeader
	$config
	$dieopenmsg
	$dieclosemsg
	@dim
	$startdate
	$logfilename
	$enddate
	%monthnr
	$n
	%lookup
);

our @EXPORT_OK = ();
our %EXPORT_TAGS = ();

our ( 
	%monthnr, $dieopenmsg, $dieclosemsg, $q, $indexpage, $ROOT, %argument,
	$curroot, $parent, $webdir, $logfilename, $verbose, $ndimension, $subsetargs,
	$maxdim, @dim, @DimOrg, $startdate, $enddate, $starttime, $previnterval, $elapsed,
	$htmlheader,$n, $pagecount, $subsetvector, $lookupargs, $lupdns, %lookup, $lookupcount,
	%dnscache, $chits, $qok, $qerrs, $noip
	);

%monthnr = (
	"Jan" => "01",
	"Feb" => "02",
	"Mar" => "03",
	"Apr" => "04",
	"May" => "05",
	"Jun" => "06",
	"Jul" => "07",
	"Aug" => "08",
	"Sep" => "09",
	"Oct" => "10",
	"Nov" => "11",
	"Dec" => "12"
);

$dieopenmsg = "Can't open file $!";
$dieclosemsg = "Can't close file $!";

@dim = ();
%lookup = ();
%dnscache = ();
$lookupcount = $pagecount = 0;

$q = new CGI ('');

no strict 'refs';

###############################
# SetCommonParameters : Sets common parameters for reporting modules
#
sub SetCommonParameters {
	
	$curroot = cwd;
	$indexpage = "index.html";
	$ROOT = "ROOT";
	@DimOrg = @dim;	# keep original copy of attribute list
	# level 0 hash
	$parent = { };

	$webdir = $config->webdir;
	$logfilename = $config->logfile;
	$verbose = $config->verbose;
	$ndimension = $config->dimension;
	$subsetargs = $config->subset;
	$subsetvector = $config->singlevector;
	$lookupargs = $config->lookup;

	VerifyParameters ();
	CheckSubsetList ();
	CheckLookupList ();	
	
	$startdate = $enddate = "";
		
	ProcessVerboseInfo ('init') if $verbose;	

}

###############################
# ProcessVerboseInfo : Generates/initializes verbose info
# One parameter : vcmd
#	'init' - initilaize counters
# 	'count', 'lookup', 'page' - process counter
# one optional parameter:
#	vcnt - counter value
#
sub ProcessVerboseInfo  {

	my ($vcmd, $vcnt) = @_;

	# Initialize counters	
	if ($vcmd eq "init") {	
		$starttime = time;
		$previnterval = 0;
		return;
	}

	$elapsed = time - $starttime; # elapsed time in seconds
		
	# Print only once for each verbose interval
	if (($elapsed != $previnterval) && (($elapsed % $config->FEEDBACKSEC) == 0)) {
		my $ltxt = ($vcmd eq 'count') ? 'lines' : ($vcmd eq 'lookup') ? 'lookups' : 'pages';
		printf "%d (%d $ltxt per sec)\n", $vcnt, $vcnt / $elapsed;
		$previnterval = $elapsed;
	}
	
}

###############################
# GenerateTables : Generates counter tables
# One optional parameter : increment count, 1 if not specified
#
sub GenerateTables {

	my $count = shift;
	
	if (!defined $count) { $count = 1; }
	
	if ($subsetvector) {
		UpdateVectorCounters ($count);
	} else {
		UpdateCounters ($ROOT, \@dim, 0, $parent, $count);
	}

	ProcessVerboseInfo ('count', $.) if $verbose;
}

###############################
# UpdateCounters: Updates counter tables recursively
# 	Five parameters -
#		currattr - current attribute,
#		attrarr - attribute array
#		dim - current dimension
#		parent - parent hash pointer
#		count - counter value
#
sub UpdateCounters {

	my ($currattr, $attrarr, $dim, $parent, $count) = @_;
	return if ($dim > $ndimension);	
	my $newparent = $parent->{$currattr}{$$currattr};
	
	if (!defined $newparent) {
		
		$newparent = $parent->{$currattr}{$$currattr} = 
				{
					"count" => $count,	# Number of occurences
					"directory" => sprintf ("%08d", int (rand($config->MAX_RANDIR))), # Random directory name
					"ref" => { }	# Reference to structures at lower dimensions
				};
				
	} else {		
		$newparent->{"count"} += $count;		
	}
		
	$dim++;
	my @dimwork = @$attrarr;
	
	for (1 .. scalar @dimwork) {
	
		my $attrval = shift @dimwork;
		UpdateCounters ($attrval, \@dimwork, $dim, $newparent->{"ref"}, $count);		
		push @dimwork, $attrval;
		
	}
		
}

###############################
# UpdateVectorCounters: Updates counter tables for a vector 
# 	Two parameters -
#		count - counter value
#
sub UpdateVectorCounters {

	my ($count) = @_;
	
	my $currattr = $ROOT;	
	my $workparent = $parent;	
	my $i = 0;
	
	do {	
		my $newparent = $workparent->{$currattr}{$$currattr};
	
		if (!defined $newparent) {
		
			$newparent = $workparent->{$currattr}{$$currattr} = 
				{
					"count" => $count,	# Number of occurences
					"directory" => sprintf ("%08d", int (rand($config->MAX_RANDIR))), # Random directory name
					"ref" => { }	# Reference to structures at lower dimensions
				};
				
		} else {		
			$newparent->{"count"} += $count;		
		}
		
		$workparent = $newparent->{"ref"};
		$currattr = $dim [$i++];
				
	} until ($i > $ndimension);
		
}

###############################
# GenerateReport: Generates logrep report based on counting tables
#
sub GenerateReport {

	my ($sumtitle, $suminfo) = @_;

	ProcessVerboseInfo ('init') if $verbose;

	# Run a lookup update if some lookups are defined
	if (scalar %lookup > 0) {
		LookupUpdate (1, $parent->{$ROOT}{$ROOT}{"ref"}); 
	}
	# Generate HTML header
	$htmlheader = GenerateHtmlHeader ($sumtitle,$suminfo);

	ProcessVerboseInfo ('init') if $verbose;
	
	# Write details pages
	WriteDetailsPages (1, $parent->{$ROOT}{$ROOT}{"ref"}, $parent->{$ROOT}{$ROOT}{"count"}, $webdir, '');
	
}

###############################
# LookupUpdate : 
#	Updates  values in counting tables by values from lookup functions
#	two parameters -
#		dim - current dimension
#		parent - pointer to count tables for the current level
#
sub LookupUpdate {

	my ($dim, $parent) = @_;

	ProcessVerboseInfo ('lookup', $lookupcount) if $verbose;

	$dim++;

	# lookup for specified lookup parameters
	foreach my $currattr (keys %$parent) {

		next if (! exists $lookup {$currattr});

		my $ltype = $lookup {$currattr};
		my $currattrval = $parent->{$currattr};
		my $xcount = 0;	# number of lookup-ed values, used by algorithm for throttling

		# Perform lookup update for values of the lookup-attribute
		foreach my $currattr2 (sort { $currattrval->{$b}{"count"} <=> $currattrval->{$a}{"count"} } keys %$currattrval) {
			
			last if ($config->MAX_X_COUNT > 0 && $xcount == $config->MAX_X_COUNT); # Throttling lookup			
			$xcount++;
				
			my $newparent = $currattrval->{$currattr2};
			# lookup value, can be undefined or null string
			$newparent->{"lookup"} = GetLookupValue ($currattr2, $ltype);

			next if ($dim > $ndimension);
	
			LookupUpdate ($dim, $newparent->{"ref"}); 	# Traverse recursively		
		}

	}

}

###############################
# GetLookupValue : 
#	Returns a lookup value from a lookup method. Undef if something is wrong or incomplete
#	two parameters -
#		lattr - attribute value to lookup
#		ltype - lookup type (only -dns- at the moment)
#
sub GetLookupValue {

	my ($lattr, $ltype) = @_;

	return undef if ($ltype ne 'dns');	# only dns

	if ($lookupcount == 0) {
		# Create a DNS resolver object
		if ($config->dns ne "<undef>") {	# get dns server lsit from command line?
			my @arr = split /,/, $config->dns;
			$lupdns = Net::DNS::Resolver->new( nameservers => \@arr );
		} else {
			$lupdns = Net::DNS::Resolver->new ();
		}
		# initialize cache
		%dnscache = ();
	}

	$lookupcount++;

	# We do only reverse lookup
	return undef if (!($lattr =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/));
	# Do we have a cache hit here ?
	return $dnscache {$lattr} if exists $dnscache {$lattr};

	my $query = $lupdns->send ($lattr);	# Send a DNS query
	if ($query) {
		foreach my $rr ($query->answer) {
			next unless $rr->type eq "PTR";	# Only interested in first PTR record
			$dnscache {$lattr} = $rr->rdatastr;
			return $rr->rdatastr;
		}
	}
 
	return ($dnscache {$lattr} = ''); # no success with query, return cache as well
}

###############################
# WriteDetailsPages : 
#	Generates html pages recursively
#	Four parameters -
#		dim - current dimension
#		parent - pointer to count tables for the current level
#		parentsum - number of occurences for parent level (sum)
#		repdir - Current report directory
#		textarr - includes names and values of previous level table entries
#
sub WriteDetailsPages {

	my ($dim, $parent, $parentsum, $repdir, @textarr) = @_;

	ProcessVerboseInfo ('page', $pagecount) if $verbose;
	
	mkdir $repdir if (! -e $repdir);	# Make current report directory
	my $prevdir = cwd;
	chdir $repdir;
	
	# Generate file name for details pages at 1st level
	if ($dim == 1) {
	
		open my $fp, "> $indexpage" or die $dieopenmsg;
	
		print $fp $htmlheader;
		print $fp $q->hr;
		print $fp '<table border="1">';

		my $onlyfirst = 0;
	
		foreach my $currattr (@dim) {

			# Make link if qualified (only 1st element for subset vectors)
			my $link = (($subsetvector && !$onlyfirst) || (!$subsetvector)) ? 
				($q->a ({href=>"$currattr/index.html"}, $currattr)) : 
				($q->font({color=>"808080"}, $currattr));
		
			$onlyfirst = 1 if ($subsetvector);
			print $fp "<tr>", $q->td ( $q->div ({align=>"center"}, $q->h3 ($link))), "</tr>";
		}

		# Write unprocessed attributes too (may be anoption in the future
		foreach my $currattr (@DimOrg) {
				
			next if grep (/$currattr/, @dim);	# Skip processed attributes
			
			my $link = $q->font({color=>"808080"}, $currattr);
			print $fp "<tr>", $q->td ( $q->div ({align=>"center"}, $q->h4($link))), "</tr>";
		}
	
		print $fp "</table>";	
		print $fp	$q->end_html;	
		close $fp or die $dieclosemsg;	
	}
	
	$dim++;
	
	foreach my $currattr (keys %$parent) {
	
		# Number of entries at next level, no graph will be produced if only 1 entry
		my $nentry = scalar keys %{$parent->{$currattr}};
		
		# Generate file name for details pages
		my $currdir = cwd;
		mkdir "$currattr" if (! -e "$currattr");	# make directory for current dimension
		chdir $currattr;
		
		open my $fp2, "> $indexpage" or die $dieopenmsg;
		print $fp2 $htmlheader;
		
		foreach my $elem (@textarr) { print $fp2 $q->p ($q->h3 ($elem)) };
		print $fp2 $q->hr;

		# Generate html output for Top N graph if number of entries ae more than 1 
		if ($nentry > 1) {
			print $fp2 
					$q->h3 (
						"Top graph for ",
						$q->font ( {color=>"#800000"}, 
							$q->em (ucfirst ($currattr))
						)
					),
					$q->p (
						$q->img ({src=>"$currattr.png",align=>"CENTER"})
					), 
					$q->hr;
		}
				
		print $fp2 
			'<table border="1">',
			"<tr>",
			$q->td ($q->b ("$currattr")),
			$q->td ($q->b ("Count")),
			$q->td ($q->b ("Ratio (%)")),
			"</tr>";

		my $currattrval = $parent->{$currattr};
		
		# Generate html output for table entries sorted by sum of occurences
		my $xcount = 0;
		
		# initialize graph array
		my @graphdata = undef;
		my $graphxn = 0;
		
		foreach my $currattr2 (sort { $currattrval->{$b}{"count"} <=> $currattrval->{$a}{"count"} } keys %$currattrval) {
			
			last if ($config->MAX_X_COUNT > 0 && $xcount == $config->MAX_X_COUNT); # Throttling output
			
			$xcount++;
				
			my $newparent = $currattrval->{ $currattr2 };

			# Take lookup value into consideration
			my $lupval = $newparent->{"lookup"};
			my $lcurrattr2 = (defined $lupval && $lupval ne "") ? $lupval : $currattr2; 
			my $repdir2 = $newparent->{"directory"}; # Get directory name for attribute value			
			my $count2 = $newparent->{"count"};			
			my $ratio = sprintf "%.2f", ($newparent->{"count"} / $parentsum) * 100;
		
			my $link = '';
			if ($dim <= $ndimension) {
				foreach my $tmpattr (keys %{$newparent->{'ref'}}) {
					$link .= $q->td ($q->a ({href=>"$repdir2/$tmpattr/index.html"}, $tmpattr));
				}
			}
			
			print $fp2 
				"<tr>", 
				$q->td ($lcurrattr2), 
				$q->td ($count2),
				$q->td ($ratio),
				$link,
				"</tr>";
				
			# Generate graph data
			if ($nentry > 1 && $graphxn < $config->MAXTOPCOUNT) {
				
				$graphdata [0][$graphxn] = GraphString ($lcurrattr2);
				$graphdata [1][$graphxn] = $newparent->{"count"};
			
				$graphxn++;
			}
			
			next if ($dim > $ndimension);
			
			push @textarr, "$currattr : $lcurrattr2";
			my $currdir2 = cwd;
			WriteDetailsPages ($dim, $newparent->{"ref"}, $newparent->{"count"}, $repdir2, @textarr);
			pop @textarr;
			chdir $currdir2;
						
		}
		
		# Print sum information at the end
		print $fp2 
				"<tr>", 
				$q->td ($q->b("SUM")), 
				$q->td ($q->b($parentsum)),
				"",
				"</tr>";
				
		print $fp2 "</table>";		
		print $fp2	$q->end_html;
		
		close $fp2 or die $dieclosemsg;
		$pagecount++;
		
		# generate graph
		if ($nentry > 1) {
		  	my $graph = GD::Graph::bars->new($config->GRAPH_SIZE_X, $config->GRAPH_SIZE_Y) or die GD::Graph->error;
			$graph->set( 
				x_labels_vertical => "1",
				y_label => "Count", 
				title => "$currattr - Top $graphxn", 
				dclrs => ["dblue"]) or die $graph->error;		

			my $gd = $graph->plot(\@graphdata) or die $graph->error;
			open my $imgfp, "> $currattr.png" or die "Problems with creating graph image -$!"; 
			binmode $imgfp;
			print $imgfp $gd->png if (defined $gd);
			close $imgfp;
		}
		
		chdir $currdir;
				
	}
	
	chdir $prevdir;

}

###############################
# GenerateHtmlHeader : 
#	Generates a standard html heeader
#	Optional two arguments for summary information
#	sumtitle, suminfo
#
sub GenerateHtmlHeader {

	my ($sumtitle, $suminfo) = @_;

	my $header = $q->start_html(
		-title=>"LOGREP - Log file reporting system",
		-style=>"
A:hover {color : Green;}
H1 {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif; font-size : x-large;}
H2 {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif; font-size : large;}
H3 {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif; font-size : medium;}
BODY {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif;	font-size : 12; color : #000080; background-color : #FFA07A;}
TABLE {font-family : Verdana, Geneva, Arial, Helvetica, sans-serif; font-size : 12; color : #000080;}
");

	return $header if (!(defined $sumtitle && defined $suminfo));

	#Generate summary header
	my $summaryheader = $q->h1 ($sumtitle);	
	
	$summaryheader .=
		$q->p (	
			$q->b ("Filename: "),
			"$logfilename ($n lines, from $startdate to $enddate)"
		);
	
	foreach my $summary (keys %$suminfo) {

		$summaryheader .=
			$q->p (
				$q->b ($summary),
				$suminfo->{$summary}
			);
	}
	
	$summaryheader .=
		$q->p (
			"Number of analyzed dimensions is ", $q->b ("$ndimension."),
	);
	
	$summaryheader .=
		$q->p (
			"This report is created on ", $q->b (scalar localtime),
	);
	
	return "$header\n$summaryheader";

}

###############################
# GraphString :
#	Formats graph strings longer than maximum allowed length
#	Parameter: String to trim
#	Return : Formatted string
sub GraphString {

	my $string = shift;

	return $string if (length ($string) <= $config->MAXGRAPHSTRLEN);
	
	$n = ($config->MAXGRAPHSTRLEN - 4) / 2;
	
	return sprintf '%s[..]%s', substr ($string, 0, $n), substr ($string, -$n);
}

###############################
# CheckSubsetList :
#   Check if a subset list is specified and verifies contents of the subset list. 
#  If everything is Ok, subset list becomes the new complete list.

sub CheckSubsetList {

	my $subsetres = 0;	# No subset

	if ($subsetargs ne "<undef>") {{	# double brackets for last command!!
	
		$subsetres = 1; # Subset list is specified. Must be verified!!
	
		my $ndim = @dim; # get number of parameters in the original complete list
	
		my @arr = split /,/, $subsetargs, $ndim;
	
		my $narr = @arr;
		last if ($narr == 0); # Don't continue if subset list is empty!
		
		# Check if subset is a really subset of the complete list
		my $nok = 0;
		foreach my $x (@arr) { 
			$nok++ if (grep (/$x/, @dim))
		 };
		last if ($narr != $nok); # Don't continue if subset list couldn't be verified
		
		@dim = @arr; # New array is qualified and replaces the complete list

		# Take user specified dimension into account if specified
		my $maxdim = scalar @dim;

		if ($maxdim < $ndimension && !$subsetvector) {
			$ndimension = $maxdim;
			print "New number of dimensions is $maxdim.\n" if $verbose;
		}
		
		$subsetres = 0; # subset is a complete list now!
	}}
	
	if ($subsetres) { # Wrong subset!!
	
		my $lststr = "";
		foreach my $x (@dim) {$lststr .= "$x,"}; 
		chop $lststr;
		
		print "Wrong subset parameter list : $subsetargs.\nA complete list of\n  $lststr\n is assumed\n";
	}
}

###############################
# VerifyParameters :
# 	Verifies command line parameters
sub VerifyParameters {
	
	my $nerr = 0;
	
	if (!(-e $logfilename)) { print STDERR "Log file $logfilename does not exist.\n"; $nerr++; };
	if (!($ndimension =~ /^\d+$/)) { print STDERR "Dimension ($ndimension) is not an integer.\n"; $nerr++ };
	
	exit 1 if ($nerr > 0);	

}

###############################
# CheckLookupList :
#   Check if a lookup list is specified and verifies contents of the that list. 

sub CheckLookupList {

	if ($lookupargs ne "<undef>") {{	# double brackets for last command!!
	
		my @arr = split /[,:]/, $lookupargs;
		
		# Check if lookup pairs are valid for this run
		my $lcnt = scalar @arr;

		for (my $i; $i < $lcnt; $i+=2) {
			
			my $lattr = $arr [$i];
			my $luptype = $arr [$i+1];
			
			print "Lookup pair : $lattr, $luptype\n" if $verbose;

			# Attribute must be available and we have currently only 'dns' as lookup
			if (!(grep (/$lattr/, @dim) && $luptype eq "dns")) {
				print "Wrong lookup list : $lookupargs.\n";
				return 1;
			}

			$lookup {$lattr} = $luptype;	# populate lookup hash
			
		 };
		
	}}
	
}

# Module end!!
1;

