package Logrep::Config;

#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2003 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

=head1 NAME

Logrep::Config - Process & maintain logrep configuration

=head1 SYNOPSIS

    use Logrep::Config;

    LoadConfigFile ("logrep.conf");

    my $logsfile = ExpandParVal ($lclient, "snapshot_server_logfile");
    my $webdir = ExpandParVal ($lclient, "snapshot_server_webpath");

    MakePathAvailable ($logsfile); 	# Make server path available if necessary
    MakePathAvailable ($webdir);	# Make logrep report path available if necessary

=head1 DESCRIPTION

This module allows processing and maintenance of configuration information for logrep 
programs. It automatically initializes a configuration context and can load settings 
from a configuration file. Settings from a file are automatically used to update 
elements like system defaults, client types, clients and favorite sets. Logrep::Config
supports parameter expansion and comes with some handy routines for that purpose.

=over 4

=item LoadConfigFile (FILENAME)

Updates logrep configuration with settings from a configuration file. If no file is given,
B<LoadConfigFile> tries to use file C<logrep.conf> in the same directory as the module itself,
or the file name specified with switches C<-f> or C<--configfile> in the command line.

=item ExpandParVal (HASHNAME, PARAMETER)

Expands pseudo values in C<PARAMETER> defined in the hash C<HASHNAME>. 

Following pseudo values are supported:

=over

=item C<$COUNT> - C<snapshot_count> value for a client

=item C<$LOGFILE> - C<logfile> value for a client

=item C<$LOGREPHOME> - Logrep home directory

=item C<$CLIENT> - Client name

=item C<$YYYY> - 4-digit year

=item C<$MM> - 	2-digit month number

=item C<$DD> - 	2-digit day number

=back

=item MakePathAvailable (PATH)

Makes sure that C<PATH> physically exist by creating all directories within the path.

=back

=cut

###########################################################

use strict "vars";
use Exporter;

use FindBin qw($Bin);
use File::Path;
use File::Basename;
use lib "$Bin/..";
use AppConfig qw (:argcount);

# Module definition globals
#
our $VERSION = "1.0.0";
our @ISA = qw(Exporter);
our @EXPORT = qw (
	&LoadConfigFile
	&ExpandParVal
	&MakePathAvailable
);

our @EXPORT_OK = qw (
	%systemdefault 
	%client
	%favoriteset
	$config 
);

our %EXPORT_TAGS = ();

our $platform = lc $^O;	# linux, mswin32

# Platform specific location of configuration file
our %confpath = (
	'linux' => "/etc/logrep",
	'mswin32' => "$Bin/../etc"
);

# Platform specific location of logrep log files
our %logpath = (
	'linux' => "/var/log/logrep",
	'mswin32' => "$Bin/../log"
);

# Platform specific data path
our %datapath = (
	'linux' => "/var/logrep",
	'mswin32' => "$Bin/../data"
);

# Logrep configuration globals
#
our %client = ();
our %favoriteset = ();
our %clientparameter = ();
our %favsetparameter = ();
our %clienttype = ();
our $config = '';
our %systemdefault = ();

# Initialize a configuration context
#
InitConfig ();

#######################################
#   LoadConfigFile : loads and updates logrep configuration from a file
#   Filename as an optional argument. Configuration parameter 'configfile' as default..
#
sub LoadConfigFile {

	my $cfile = (scalar @_ > 0) ? shift : $config->configfile;

	print "Reading configuration from $cfile ..\n" if $config->debug;

	# Read configuration file
	$config->file($cfile) || die ();

	UpdateSystemConfig ();	# Update system defaults
	UpdateClientConfig ();	# Update Client configuration
	UpdateFavsetConfig ();	# Update Favorite Set information

}

#######################################
#   ExpandParVal : expands a logrep configuration parameter.
#   Two required parameters : 
#	hashname - where parameter is defined
#       parameter - logrep configuration parameter
#
sub ExpandParVal {

	my ($hashname, $parameter, $favsetsw) = @_;

	# If favorite switch is defined, then we return an array of fav set parameters
	return @{$$hashname {$parameter}} if (defined $favsetsw);

	my $phash = \%$hashname;

	my $parval = $phash->{$parameter};

	if ( $parval =~ /\$COUNT/) { 
		my $tmprep = $phash->{"snapshot_count"};
		$parval =~ s/\$COUNT/$tmprep/g;
	};

	if ( $parval =~ /\$LOGFILE/) { 
		my $tmprep = $phash->{"logfile"};
		$parval =~ s/\$LOGFILE/$tmprep/g;
	};

	if ( $parval =~ /\$DATAPATH/) { 
		my $tmprep = $config->datapath ();
		$parval =~ s/\$DATAPATH/$tmprep/g;
	};

	if ( $parval =~ /\$CONFPATH/) { 
		my $tmprep = $config->confpath ();
		$parval =~ s/\$CONFPATH/$tmprep/g;
	};

	if ( $parval =~ /\$LOGPATH/) { 
		my $tmprep = $config->logpath ();
		$parval =~ s/\$LOGPATH/$tmprep/g;
	};

	if ( $parval =~ /\$LOGREPHOME/) { 
		$parval =~ s/\$LOGREPHOME/$Bin\/\.\./g
	};

	if ( $parval =~ /\$CLIENT/) {
		$parval =~ s/\$CLIENT/$hashname/g
	};

	if ( $parval =~ /\$YYYY/) {
    		my (undef,undef,undef,undef,undef,$year,undef,undef,undef) = localtime(time);
		$year += 1900;
		$parval =~ s/\$YYYY/$year/g;
	};

	if ( $parval =~ /\$YY/) {
    		my (undef,undef,undef,undef,undef,$year,undef,undef,undef) = localtime(time);
		$year = sprintf("%02d", $year % 100);
		$parval =~ s/\$YYYY/$year/g;
	};
	
	if ( $parval =~ /\$MM/) {
    		my (undef,undef,undef,undef,$month,undef,undef,undef,undef) = localtime(time);
		$month = sprintf ("%02d", $month+1);
		$parval =~ s/\$MM/$month/g;
	};

	if ( $parval =~ /\$DD/) {
    		my (undef,undef,undef,$mday,undef,undef,undef,undef,undef) = localtime(time);
		$mday = sprintf ("%02d", $mday);
		$parval =~ s/\$DD/$mday/g;
	};

	if ( $parval =~ /\$NTEVENT/) { 
		my $tmprep = $phash->{"eventtype"};
		$parval =~ s/\$NTEVENT/$tmprep/g;
	};

	if ( $parval =~ /\$SYSTEMROOT/) { 
		my $tmprep = $ENV {"SYSTEMROOT"};
		$parval =~ s/\$SYSTEMROOT/$tmprep/g;
	};

	return $parval;
}

#######################################
#   MakePathAvailable: makes sure that specified path does physically exists
#   Requires path name as a parameter.
#
sub MakePathAvailable {

	my $path = dirname (shift);
	mkpath ($path) if (! -e $path);
}

######################################
#   UpdateSystemConfig: Updates system-wide defaults (global hash %systemdefault)
#   with values loaded from configuration file.

sub UpdateSystemConfig {

	# Update logrep system defaults
	%systemdefault = $config->varlist ("^logrep_system_default_", 1);

	if ($config->debug) {
		print "System wide defaults:\n\n";
		while (my ($key, $value) = each %systemdefault) {
			print "    $key --> $value\n";
		}
	}
}

######################################
#   UpdateClientConfig: Updates client specific configuration with values from a configuration file. 
#   Populates global hash %client and make sure that each client has all parameters defined and verified.
#
sub UpdateClientConfig {

	# Check if we have a complete set of system wide defaults
	foreach my $clnpar (keys %clientparameter) {

		# Supply a value from a pre-defined variable if not defined in configuration file 
		$systemdefault {$clnpar} = $config->get("$clnpar") if (!defined $systemdefault {$clnpar});

		# Raise error and quit if there is still no definition for a system wide default value.
		if (!defined $systemdefault {$clnpar}) {
			die "A system wide default value for parameter $clnpar must be defined.\n";
		}
	}

	# Get client type defaults
	foreach my $lsys (keys %clienttype) {

		my $prefix = "^logrep_" . $lsys . "_default_";
		%{$clienttype {$lsys}} = $config->varlist ($prefix, 1);

	}

	# Get user defined logrep client names
	%client = $config->varlist ("^logrep_clients_", 1);

	# Get, complement and verify client data
	foreach my $lcln (keys %client) {

		my $prefix = "^" . $lcln . "_";
		%$lcln = $config->varlist ($prefix, 1);
		my $lclnhash = $$lcln;

		# ipaddress and client type must be specified for each client by the user.
		for my $clnpar ("ipaddress", "type") {
			if (!defined $$lcln {$clnpar}) { 
				die "Configuration failure - Parameter $clnpar must be defined for client $lcln.\n"
			}
		}
	
		my $clntype = $$lcln {"type"};	# Get client type

		# make sure that every parameter is defined and verified at client level
		foreach my $lclnpar (keys %clientparameter) {

			# use client type default first if the parameter is not defined at client level
			$$lcln {$lclnpar} =  $clienttype{$clntype}->{$lclnpar} if (!defined ($$lcln {$lclnpar}));

			# use system wide default if still not defined
			$$lcln {$lclnpar} = $systemdefault {$lclnpar} if (!defined ($$lcln {$lclnpar}));

			# Raise error and quit if parameter is still not defined
			if (!defined ($$lcln {$lclnpar})) {
				die "Client parameter $lclnpar for client $lcln is not defined at all.\n";
			}

			# verify client parameter value
			my $clnpararr = $clientparameter {$lclnpar};

			if (defined $clnpararr->[0]) {

				# Call parameter-specific verification routine
				my $clnparver = &{$clnpararr->[0]} ($$lcln {$lclnpar}, $clnpararr->[1]);
			
				die "Parameter $lclnpar value ($$lcln{$lclnpar}) for client $lcln could not be verified.\nMust be a(n) $clnpararr->[1] value.\n"
					if !$clnparver;
			}
		}
	}

}

######################################
#   UpdateFavsetConfig: Updates favorite set specific configuration with values from a configuration file. 
#   Populates global hash %favoriteset and verifies values.
#
sub UpdateFavsetConfig {

	# Get user defined logrep favoriteset names
	%favoriteset = $config->varlist ("^logrep_favoritesets_", 1);

	# Get, complement and verify favorite set data
	foreach my $lfavset (keys %favoriteset) {

		my $prefix = "^" . $lfavset . "_";
	
		# We keep parameters in a temporary hash before normalizing
		my $tmphashname = "tmp$lfavset";
		%$tmphashname = $config->varlist ($prefix, 1);

		# Normalize parameter names and values
		# Normalization is done  by stripping off sequence number
		# and pushing parameter values into an array named as parameter name

		while (my ($lfavsetpar, $lfavsetval) = each %$tmphashname) {

			# Strip trailing number from favset parameter to determine parameter type
			my ($lfavsetparn) = ($lfavsetpar =~ /^(\w+)\d+$/);

			# check if parameter is pre-defined
			die "Invalid parameter name ($lfavsetparn) for favorite set $lfavset.\n"
				if !exists $favsetparameter {$lfavsetparn};

			# Put favset parameter value in a array
			push (@{$$lfavset{$lfavsetparn}}, $lfavsetval);

			# verify favoriteset parameter value
			my $favsetpararr = $favsetparameter {$lfavsetparn};

			if (defined $favsetpararr->[0]) {
	
				# Call parameter-specific verification routine
				my $favsetparver = &{$favsetpararr->[0]} ($lfavsetval, $favsetpararr->[1]);
			
				die "Parameter $lfavsetparn value ($lfavsetval) for favorite set $lfavset could not be verified.\nMust be a(n) $favsetpararr->[1] value.\n"
					if !$favsetparver;
			}
		}
	}
}

#######################################
#   InitConfig : Creates a configuration context.
#
sub InitConfig {

	# available client parameters
	%clientparameter = (

		"snapshot_count" => [ 
			\&WhatIsThat, 
			"number", 
			"Number of lines from the bottom of a log file (snapshot reports)."
		],

		"snapshot_command" => [ 
			undef, 
			undef, 
			"Remote command for data extraction (snapshot reports)."
		],

		"snapshot_interval" => [
			\&WhatIsThat,
			"number",
			"Time interval in minutes for generation of snapshot reports."
		],

		"snapshot_server_logfile" => [
			undef,
			undef,		
			"Server copy of extracted log file (snapshot reports)."
		],

		"snapshot_server_webpath" => [
			undef,
			undef,
			"Location of snapshot reports."
		],

		"full_command" => [
			undef,
			undef,
			"Remote command for data extraction (full reports)."
		],

		"full_schedule"	=> [
			undef,
			undef,
			"When full reports will be generated."
		],

		"full_server_logfile" => [
			undef,
			undef,
			"Server copy of extracted log file (full reports)."
		],

		"full_server_webpath" => [
			undef, 
			undef,
			"Location of full reports."
		],

		"localuser" => [
			undef,
			undef,
			"Name of the local user at the clients."
		],

		"keyfile" => [
			undef,
			undef,
			"Private key for SSH public key authentication."
		],

		"feedback_seconds" => [
			\&WhatIsThat,
			"number",
			"Time interval in seconds between progress messages if verbose option is turned on."
		],

		"random_dirlimit" => [
			\&WhatIsThat,
			"number",
			"Maximum number of directories with random names."
		],

		"favoriteset_server_webpath" => [
			undef,
			undef,		
			"Location of favoriteset documents."
		],

		"logrep_logfile" => [
			undef,
			undef,
			"Log file for logrep activity."
		],

		"max_topcount" => [
			\&WhatIsThat,
			"number",
			"Maximum number of elements in a logrep graph."
		],

		"max_graphstring" => [
			\&WhatIsThat,
			"number",
			"Maximum number of characters in a logrep graph string."
		],

		"max_tablecount" => [
			\&WhatIsThat,
			"number",
			"Maximum number of elements in a logrep table."
		],

		"graph_width" => [
			\&WhatIsThat,
			"number",
			"Graph width in pixels."
		],

		"graph_height" => [
			\&WhatIsThat,
			"number",
			"Graph height in pixels."
		],

		"logfile" => [
			undef,
			undef,
			"Name of source log file."
		],

		"ipaddress" => [
			\&WhatIsThat,
			"ip",
			"Logrep client IP address."
		],

		"type" => [
			\&WhatIsThat,
			"clienttype",
			"Logrep client type"
		],

		"attributes" => [
			undef,
			undef, 
			"Comma separated attribute list for report generation."
		],

		"dimensions" => [
			\&WhatIsThat,
			"number",
			"Number of dimensions for logrep analysis."
		],

		"singlevector" => [
			undef,
			undef,
			"Whether logrep analysis will be limited to one sequence/vector"
		],

		"eventtype" => [
			\&WhatIsThat,
			"eventtype",
			"NT Event type"
		],

	);

	# available favorite set parameters
	%favsetparameter = (

		"favoriteset" => [ 
			\&WhatIsThat, 
			"favoriteset", 
			"Favorite set name."
		],

		"graph" => [ 
			undef, 
			undef, 
			"Favorite set graph element."
		],

		"report" => [
			undef,
			undef,
			"Favorite set report element."
		],

		"client" => [
			\&WhatIsThat,,
			"client",		
			"Favorite set client element."
		],
	);

	# Logrep supported client types
	%clienttype = (

		"snort"		=> {},
		"squid" 	=> {},
		"postfix"	=> {},
		"apache"	=> {},
	 	"iis"		=> {},
 		"iscan"		=> {},
		"ntevent"	=> {},
		"syslog"	=> {},
		"firewall1"	=> {},
		"exchange"	=> {},
		"ipchains"	=> {},
		"pix"		=> {},
		"iptables"	=> {},
		"netscreen"	=> {},
		"wtmp"		=> {},
		"oracle"	=> {},
	);

	# Create a config object, we require one parameter as default
	$config = AppConfig->new(
		{
			CREATE => 1,
			GLOBAL => {
				ARGCOUNT => ARGCOUNT_ONE
			},
			DEBUG => 0
		}
	);

	# Define some hard defaults
	$config->define ("FEEDBACKSEC",		{DEFAULT => 3,		ALIAS => "feedback_seconds" } );
	$config->define ("MAXTOPCOUNT",		{DEFAULT => 15,		ALIAS => "max_topcount"});
	$config->define ("MAXGRAPHSTRLEN",	{DEFAULT => 30,		ALIAS => "max_graphstring"});
	$config->define ("MAX_X_COUNT",		{DEFAULT => 50,		ALIAS => "max_tablecount"});
	$config->define ("MAX_RANDIR",		{DEFAULT => 99999,	ALIAS => "random_dirlimit"});
	$config->define ("GRAPH_SIZE_X",	{DEFAULT => 400,	ALIAS => "graph_width"});
	$config->define ("GRAPH_SIZE_Y",	{DEFAULT => 400,	ALIAS => "graph_height"});
	$config->define ("ATTRLIST",		{DEFAULT => "*",	ALIAS => "attributes"});
	$config->define ("MAXDIM",		{DEFAULT => 2,		ALIAS => "dimensions"});
	$config->define ("SINGLEVEC",		{DEFAULT => 0,		ALIAS => "singlevector"});
	$config->define ("LOGFILE",		{DEFAULT => "",		ALIAS => "logfile"});
	$config->define ("TYPE",		{DEFAULT => "",		ALIAS => "type"});
	$config->define ("IPADDRESS",		{DEFAULT => "",		ALIAS => "ipaddress"});
	
	# Definition of data path
	$config->define ("DATAPATH",
		{
			DEFAULT => "$datapath{$platform}", 
			ALIAS => "datapath"
		}
	);

	# Definition of configuration path
	$config->define ("CONFPATH",
		{
			DEFAULT => "$confpath{$platform}", 
			ALIAS => "confpath"
		}
	);

	# Definition of log path
	$config->define ("LOGPATH",
		{
			DEFAULT => "$logpath{$platform}", 
			ALIAS => "logpath"
		}
	);

	# Definition of configuration file
	$config->define ("LOGREPCONF",	
		{
			DEFAULT => "$confpath{$platform}" . "/logrep.conf", 
			ALIAS => "configfile|c"
		}
	);

	# Verbose option
	$config->define ("VERBOSE",	
		{
			DEFAULT => 0,
			ARGCOUNT => ARGCOUNT_NONE, 
			ALIAS => "verbose|v"
		}
	);

	# Log file name
	$config->define ("LOGFILENAME",	
		{
			DEFAULT => "<undef>", 
			ALIAS => "logfile|l"
		}
	);

	# Directory for web reports
	$config->define ("WEBDIR",	
		{
			DEFAULT => "<undef>", 
			ALIAS => "webdir|w"
		}
	);

	# subset parameter list
	$config->define ("SUBSET",	
		{
			DEFAULT => "<undef>", 
			ALIAS => "subset|s"
		}
	);

	# subset vector indicator
	$config->define ("SINGLEVECTOR",	
		{
			DEFAULT => 0,
			ARGCOUNT => ARGCOUNT_NONE, 
			ALIAS => "singlevector|r"
		}
	);

	# lookup parameter list
	$config->define ("LOOKUP",	
		{
			DEFAULT => "<undef>", 
			ALIAS => "lookup|u"
		}
	);

	# dns server list
	$config->define ("DNS",	
		{
			DEFAULT => "<undef>", 
			ALIAS => "dns"
		}
	);

	# dimension of analysis
	$config->define ("DIMENSION",	
		{
			DEFAULT => 2, 
			ALIAS => "dimension|d"
		}
	);

	# Verbose option
	$config->define ("DEBUG",	
		{
			DEFAULT => 0,
			ARGCOUNT => ARGCOUNT_NONE, 
			ALIAS => "debug"
		}
	);


	# Get command line options
	$config->args() || die ();

}

###################################
#    WhatIsThat : Verification routine for parameters from configuration file
#    Requires two arguments :
#        val  :	parameter value
#        type : parameter type
#
sub WhatIsThat {

	my ($val, $type) = @_;

	if ($type eq "number") { return ($val =~ /^\d+$/) };

	if ($type eq "clienttype") { return (exists $clienttype {$val} ) };

	if ($type eq "ip")	{ return $val =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/ };

	if ($type eq "client") { 
		my ($tmpval) = split /\s+/, $val;
		return (exists $client {$tmpval}) 
	};

	if ($type eq "eventtype") {
		my $tmpval = lc $val;
		return  (($val eq "system") || ($val eq "application") || ($val eq "security"));
	};

	if ($type eq "favoriteset") { return (exists $favoriteset {$val} ) };

	return 0;

};

# Module end!!
1;


